-- The main page that is loaded when you go to the URL for this channel.
local function indexPage()
   net.http.respond{body=
[[<html>
   <head>
      <title>Sample Web Application</title>
      <link rel="stylesheet" href="?resource=my.css"></link>
      <script type="text/javascript" src="?resource=jquery-1.10.1.min.js"></script>
      <script type="text/javascript"><!--
         /* In this script you can now use the jQuery library, since it's included above */
         $(document).ready(function(){
            $('#more').click(function(){
               jQuery.post('', {ajax:'getHello'}, function(data){
                  $(data).insertBefore($('#more')).slideDown('fast');
               });
            });
         });
      --></script>
   </head>
   <body>
      <p>Hello World</p>
      <button id="more">More...</button>
   </body>
</html>]]}
end

-- Server-side state
local ExtraHelloCount = 1

-- AJAX request handler
local function getHello(Params)
   net.http.respond{entity_type='text/plain',
      body=
[[<p class="moreHello">
   Hello Again (]]..ExtraHelloCount..[[)
</p>]]}
   ExtraHelloCount = ExtraHelloCount + 1
end

-- The functions accepts the HTTP request as a parameter.
local AjaxHandlers = {
   ['getHello']=getHello
}

local ContentTypeMap = {
   ['.js']  = 'application/x-javascript',
   ['.css'] = 'text/css',
   ['.gif'] = 'image/gif'
}

local function contentTypeFromFileName(FileName)
   local Ext = FileName:match('.*(%.%a+)$')
   if ContentTypeMap[Ext] then
      return ContentTypeMap[Ext]
   else
      return 'text/plain'
   end
end

-- Serve up web docs (.css, .js files) from the "other" files in this project.
local function serveWebDoc(Params)
   if Params.resource then
      local ResourcePath = 'other/'..Params.resource
      if iguana.project.files()[ResourcePath] then
         local F = io.open(iguana.project.files()[ResourcePath], 'rb')
         local WD = ((F and F:read('*a')) or nil)
         if F then F:close() end
         local T = contentTypeFromFileName(Params.resource)
         local HttpResponse = net.http.respond{entity_type=T, body=WD}
         iguana.logDebug(HttpResponse)
         return true
      else
         iguana.logError('Request resource "'..ResourcePath..'" was not found')
         pcall(function()
               net.http.respond{entity_type='text/plain', body=
                  'The resource requested was not found.', code=404}
             end)
      end
   end
   return false
end

-- The main entry point: all HTTP requests routed to this
-- channel come in here.
function main(Data)
   local R = net.http.parseRequest{data=Data}
   
   if serveWebDoc(R.params) then return end
   
   if AjaxHandlers[R.params.ajax] then
      local Handler = AjaxHandlers[R.params.ajax]
      if iguana.isTest() then
         Handler(R.params)
      else
         local Success, Err = pcall(Handler, R.params)
         if not Success then
            iguana.logError(tostring(Err))
            pcall(function()
                  net.http.respond{entity_type='text/json',
                     body=json.serialize{data={ErrorMessage=tostring(Err)}}}
               end)
         end
      end
      return
   end
   
   local Handler = indexPage
   if iguana.isTest() then
      Handler(R.params)
   else
      local Success, Err = pcall(Handler, R.params)
      if not Success then
         iguana.logError(tostring(Err))
         pcall(function()
               net.http.respond{entity_type='text/html', body=
                  '<html><head><title>Error</title></head><body>'..
                  '<div class="err">'..filter.html.enc(tostring(Err))..
                  '</div></body></html>', code=500}
            end)
      end
   end
end